import sys
import os
import unittest

# Add the parent directory to the system path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from geomaglib import sh_loader

class TestShLoader(unittest.TestCase):

    def _test_act_dict_to_exp_dict(self, act_dict, exp_dict):
        if "epoch" in exp_dict:
            self.assertEqual(act_dict["epoch"],exp_dict["epoch"])
        for key in exp_dict:
            if key == "epoch":
                continue
            act_arr = act_dict.get(key)
            exp_arr = exp_dict.get(key)
            for i in range(len(exp_arr)):
                self.assertAlmostEqual(act_arr[i], exp_arr[i], places=10)

    def test_sh_to_num_elems(self):
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(1), 3)
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(2), 6)
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(3), 10)
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(10), 66)
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(16), 153)
        self.assertEqual(sh_loader.calc_sh_degrees_to_num_elems(133), 9045)

    def test_num_elems_to_sh(self):
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(3),1)
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(6),2)
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(10),3)
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(66),10)
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(153),16)
        self.assertEqual(sh_loader.calc_num_elems_to_sh_degrees(9045),133)

    @unittest.skipIf(os.getenv("GITHUB_ACTIONS") == "true", "Skipping test in CI/CD since HDGM coefficient files are not allow to push in public")
    def test_load_coef(self):
        #exp_dict = {'g': [0,-29404.5, -1450.7, -2500.0, 2982.0, 1676.8, 1363.9, -2381.0, 1236.2, 525.7, 903.1, 809.4, 86.2, -309.4, 47.9, -234.4, 363.1, 187.8, -140.7, -151.2, 13.7, 65.9, 65.6, 73.0, -121.5, -36.2, 13.5, -64.7, 80.6, -76.8, -8.3, 56.5, 15.8, 6.4, -7.2, 9.8, 23.6, 9.8, -17.5, -0.4, -21.1, 15.3, 13.7, -16.5, -0.3, 5.0, 8.2, 2.9, -1.4, -1.1, -13.3, 1.1, 8.9, -9.3, -11.9, -1.9, -6.2, -0.1, 1.7, -0.9, 0.6, -0.9, 1.9, 1.4, -2.4, -3.9, 3.0, -1.4, -2.5, 2.4, -0.9, 0.3, -0.7, -0.1, 1.4, -0.6, 0.2, 3.1, -2.0, -0.1, 0.5, 1.3, -1.2, 0.7, 0.3, 0.5, -0.2, -0.5, 0.1, -1.1, -0.3], 'h': [0,0.0, 4652.9, 0.0, -2991.6, -734.8, 0.0, -82.2, 241.8, -542.9, 0.0, 282.0, -158.4, 199.8, -350.1, 0.0, 47.7, 208.4, -121.3, 32.2, 99.1, 0.0, -19.1, 25.0, 52.7, -64.4, 9.0, 68.1, 0.0, -51.4, -16.8, 2.3, 23.5, -2.2, -27.2, -1.9, 0.0, 8.4, -15.3, 12.8, -11.8, 14.9, 3.6, -6.9, 2.8, 0.0, -23.3, 11.1, 9.8, -5.1, -6.2, 7.8, 0.4, -1.5, 9.7, 0.0, 3.4, -0.2, 3.5, 4.8, -8.6, -0.1, -4.2, -3.4, -0.1, -8.8, 0.0, -0.0, 2.6, -0.5, -0.4, 0.6, -0.2, -1.7, -1.6, -3.0, -2.0, -2.6, 0.0, -1.2, 0.5, 1.3, -1.8, 0.1, 0.7, -0.1, 0.6, 0.2, -0.9, -0.0, 0.5], 'g_sv': [0,6.7, 7.7, -11.5, -7.1, -2.2, 2.8, -6.2, 3.4, -12.2, -1.1, -1.6, -6.0, 5.4, -5.5, -0.3, 0.6, -0.7, 0.1, 1.2, 1.0, -0.6, -0.4, 0.5, 1.4, -1.4, -0.0, 0.8, -0.1, -0.3, -0.1, 0.7, 0.2, -0.5, -0.8, 1.0, -0.1, 0.1, -0.1, 0.5, -0.1, 0.4, 0.5, 0.0, 0.4, -0.1, -0.2, -0.0, 0.4, -0.3, -0.0, 0.3, -0.0, -0.0, -0.4, 0.0, -0.0, -0.0, 0.2, -0.1, -0.2, -0.0, -0.1, -0.2, -0.1, -0.0, -0.0, -0.1, -0.0, 0.0, -0.0, -0.1, 0.0, -0.0, -0.1, -0.1, -0.1, -0.1, 0.0, -0.0, -0.0, 0.0, -0.0, -0.0, 0.0, -0.0, 0.0, -0.0, -0.0, -0.0, -0.1], 'h_sv': [0,0.0, -25.1, 0.0, -30.2, -23.9, 0.0, 5.7, -1.0, 1.1, 0.0, 0.2, 6.9, 3.7, -5.6, 0.0, 0.1, 2.5, -0.9, 3.0, 0.5, 0.0, 0.1, -1.8, -1.4, 0.9, 0.1, 1.0, 0.0, 0.5, 0.6, -0.7, -0.2, -1.2, 0.2, 0.3, 0.0, -0.3, 0.7, -0.2, 0.5, -0.3, -0.5, 0.4, 0.1, 0.0, -0.3, 0.2, -0.4, 0.4, 0.1, -0.0, -0.2, 0.5, 0.2, 0.0, -0.0, 0.1, -0.3, 0.1, -0.2, 0.1, -0.0, -0.1, 0.2, -0.0, 0.0, -0.0, 0.1, 0.0, 0.2, -0.0, 0.0, 0.1, -0.0, -0.1, 0.0, -0.0, 0.0, -0.0, 0.0, -0.1, 0.1, -0.0, 0.0, -0.0, 0.1, -0.0, -0.0, 0.0, -0.1], "epoch": 2020}

        #self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/WMM2020.COF", True),exp_dict)
        exp_dict = {'g': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0803, 0.1195, -0.3267, 0.0969, 0.0021, 0.0413, 0.088, -0.0256, 0.1967, -0.1968, 0.2133, 0.206, 0.0039, -0.1265, -0.0929, -0.0291, -0.1002, -0.0262, 0.0148, -0.0391, 0.313, 0.2265, -0.0101, 0.0295, -0.0588, 0.1605, 0.0957, 0.0023, -0.1489, -0.1266, 0.009, 0.0244, 0.0493, 0.025, -0.1629, 0.2308, 0.0882, 0.0413, 0.1776, -0.0499, 0.0813, 0.1821, -0.0432, 0.0425, -0.055, 0.1149, -0.0166, -0.0582, -0.0334, -0.007, 0.0265, -0.0228, 0.146, -0.024, -0.0205, 0.412, -0.1014, 0.0906, 0.0724, 0.0649, -0.0014, 0.1209, -0.0068, 0.1043, -0.0742, 0.029, -0.098, -0.0176, 0.0832, 0.1396, 0.0568, -0.031, 0.0785, -0.0505, -0.049, 0.2206, -0.1094, 0.1768, -0.1867, 0.3057, 0.2196, -0.0863, -0.0237, 0.0609, 0.0558, -0.1585, -0.0118, -0.0742, 0.054, -0.0055, 0.0113, -0.026, -0.1414, 0.0569, -0.0106, -0.1292, 0.0519, -0.1051, 0.1714, -0.036, 0.217, -0.0965, 0.0141, -0.1465, 0.0647, -0.0951, 0.07, -0.0405, -0.0611, 0.0582, 0.0364, 0.0515, -0.0851, -0.0763, 0.0103, 0.1952, 0.1953, 0.087, -0.0679, -0.1077, 0.0446, -0.15, -0.0277, 0.0706, -0.1729, 0.0184, -0.1303, 0.1832, -0.0709, -0.1158, -0.0138, 0.1137, -0.0608, 0.0666, -0.0706, -0.1467, 0.0191, 0.0007, 0.0623, 0.1107, 0.2084, 0.1249, -0.2545, 0.2171, 0.019, -0.1528, 0.0236, -0.025, -0.194, -0.2108, 0.1734, 0.1264, 0.1687, -0.2498, -0.0464, 0.0538, 0.1611, 0.0188, 0.0235, 0.0052, 0.1954, 0.121, 0.0331, 0.0675, 0.0176, -0.0428, 0.2655, 0.0324, -0.0012, -0.0916, 0.1844, 0.0624, 0.2843, -0.0868, 0.0242, 0.2173, 0.1548, 0.1242, -0.0695, -0.0567, 0.1075, 0.0558, -0.0534, 0.0267, -0.1265, 0.018, -0.0112, 0.0121, -0.0191, 0.1972, 0.0861, 0.003, 0.1842, 0.1265, -0.0193, 0.1564, -0.1564, -0.0185, 0.0973, 0.1221, 0.0034, 0.1621, 0.2124, -0.1078, 0.005, 0.0907, -0.0066, 0.2522, 0.0387, -0.0481, -0.0406, 0.0175, -0.17, -0.0428, 0.076, 0.1789, 0.1865, -0.1507, -0.0229, -0.0056, -0.1575, -0.1046, 0.2064, 0.0212, 0.1823, -0.1233, 0.1228, 0.2879, -0.147, -0.1345, 0.1328, 0.124, 0.0043, 0.0354, 0.1305, -0.0195, -0.0333, 0.0408, -0.0171, 0.028, 0.0615, -0.0469, 0.264, 0.0928, -0.072, 0.2041, -0.1443, -0.0862, 0.1131, 0.0557, -0.0521, 0.0594, 0.0568, 0.0966, 0.1114, 0.0302, 0.115, 0.0018, 0.0585, 0.0618, 0.1263, -0.1034, 0.0073, -0.0966, -0.112, -0.009, -0.0386, -0.0865, 0.1284, -0.0475, -0.1309, 0.1995, -0.0847, -0.1108, 0.1406, 0.0865, -0.1659, 0.0824, 0.1189, -0.0689, 0.0125, 0.0881, -0.0307, 0.0536, 0.1422, 0.0306, 0.0331, 0.0154, -0.0436, 0.089, -0.1638, -0.0806, 0.0643, 0.1269, 0.0464, 0.0559, -0.0263, -0.0384, -0.0683, 0.0258, 0.1123, 0.1244, 0.2222, 0.0899, 0.0609, -0.0374, -0.0902, 0.1066, 0.023, -0.0755, 0.0246, 0.1265, 0.1102, 0.0259, -0.0243, 0.1763, 0.0778, -0.0691, -0.0121, -0.0111, -0.0494, -0.063, -0.0117, -0.0589, -0.0594, -0.0074, -0.1973, 0.0615, -0.0356, 0.0465, -0.0607, 0.1473, -0.3491, -0.0767, 0.2267, -0.0339, 0.0446, 0.1976, 0.1362, 0.0247, 0.0877, 0.0476, 0.2017, 0.0604, 0.004, -0.0052, 0.1256, -0.0245, -0.0829, -0.013, 0.0073, -0.0587, 0.0479, 0.0868, 0.0576, -0.0613, 0.0466, -0.0432, 0.0262, 0.0176, -0.0197, 0.0892, 0.1446, -0.0196, 0.1127, 0.1212, -0.0004, -0.0596, -0.0161, -0.0576, -0.0525, 0.019, 0.0709, 0.2478, -0.0712, -0.2017, 0.1697, 0.002, -0.0948, -0.0486, -0.0977, 0.0049, -0.0402, -0.051, -0.1274, -0.0454, -0.0089, 0.0145, -0.0398, -0.0295, -0.048, -0.0543, -0.1043, 0.0654, -0.0329, -0.0993, -0.1111, 0.0097, -0.1214, 0.0441, -0.0727, -0.1897, -0.0134, 0.0291, 0.006, -0.0158, 0.0714, -0.2036, -0.0857, 0.0826, -0.0605, -0.2126, -0.233, 0.136, 0.0856, -0.0291, -0.0486, 0.0344, 0.1271, 0.0969, -0.0378, 0.0518, 0.018, -0.0925], 'h': [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3072, 0.1376, 0.2715, -0.068, -0.1108, -0.0608, -0.0771, 0.0069, 0.012, -0.1232, -0.0281, -0.1301, -0.1219, 0.2211, -0.0081, -0.1548, 0.0, 0.0988, -0.2699, -0.0351, -0.1108, -0.2339, -0.0264, -0.2892, 0.0184, -0.0494, -0.0621, 0.1344, 0.1234, -0.0546, -0.06, 0.1147, -0.3536, -0.0202, 0.0, 0.0527, 0.1289, -0.078, 0.0876, -0.0489, 0.0777, -0.2074, -0.1249, -0.1285, 0.163, 0.113, -0.1302, -0.0519, 0.1374, -0.0526, 0.0575, -0.0506, 0.0972, 0.0, 0.1066, -0.0108, -0.0232, 0.0976, -0.3413, -0.013, -0.1664, -0.0178, -0.0434, 0.0882, 0.1721, 0.0159, 0.0656, -0.1215, 0.0938, -0.0553, 0.0086, -0.0723, -0.0202, 0.0, 0.0208, 0.1961, 0.084, -0.3323, 0.0566, 0.0986, 0.1333, -0.1627, -0.1757, -0.0499, 0.0331, 0.0035, 0.0273, -0.1465, 0.0318, 0.0592, 0.0463, 0.1601, 0.0076, -0.0806, 0.0, 0.3891, 0.0326, 0.1697, 0.0718, -0.2128, 0.0862, -0.1371, -0.1518, -0.1376, -0.0628, 0.0085, -0.1898, 0.0136, -0.0638, -0.0592, -0.1592, -0.0881, 0.0902, 0.0028, -0.1235, -0.129, 0.0, 0.1099, -0.0847, 0.3469, 0.0933, 0.0049, -0.0437, -0.0099, -0.1964, -0.0348, 0.0976, -0.2147, -0.1258, 0.0857, -0.0333, -0.1771, -0.1342, 0.0338, 0.0153, -0.0013, 0.0603, 0.0677, 0.0479, 0.0, 0.2266, 0.1296, 0.1408, -0.1584, 0.0538, 0.0124, -0.1289, -0.1192, -0.0267, -0.038, 0.1644, -0.0748, -0.0687, 0.239, -0.0232, -0.0094, 0.0485, 0.0121, 0.0785, -0.0166, -0.1575, -0.0566, 0.0383, 0.0, -0.0441, -0.1063, 0.0081, -0.1009, 0.0636, 0.1126, 0.1178, 0.1573, -0.2175, 0.1425, 0.2016, 0.056, 0.0356, -0.0245, 0.0068, 0.1364, 0.0803, 0.0233, 0.1003, 0.0844, 0.0538, 0.0683, 0.0056, 0.0134, 0.0, 0.1434, -0.2522, 0.2275, 0.1713, 0.0029, -0.0667, 0.2158, 0.1982, 0.2417, -0.0745, 0.0567, 0.2942, 0.0816, 0.0064, -0.1225, -0.1157, 0.2584, 0.0558, -0.0533, -0.0901, -0.0846, -0.0025, -0.0713, -0.0144, 0.0022, 0.0, -0.1103, -0.3059, 0.1576, 0.2212, 0.0671, -0.1986, 0.1004, 0.0579, -0.0155, -0.0367, 0.1666, 0.0014, -0.0349, -0.0894, -0.0427, -0.2327, 0.0286, 0.0639, -0.0946, -0.0452, -0.0779, 0.0913, 0.0182, -0.0966, -0.1754, 0.1, 0.0, 0.2732, -0.1839, 0.0096, 0.1451, 0.1442, -0.1816, -0.102, -0.0176, 0.269, -0.0667, 0.005, -0.1619, 0.0247, -0.0828, -0.0142, 0.0275, 0.0074, -0.0225, -0.0882, -0.0958, -0.1353, 0.0258, 0.0524, 0.0357, 0.0731, 0.036, -0.0377, 0.0, 0.1346, -0.172, 0.0403, -0.0136, -0.0266, -0.0686, 0.2288, -0.1821, 0.1297, 0.0333, -0.0092, -0.0274, -0.0392, -0.0563, 0.0955, -0.0827, 0.0294, 0.1267, -0.0618, -0.1652, 0.0639, 0.0784, 0.0429, 0.0057, -0.0646, -0.106, 0.0038, 0.0931, 0.0, -0.1075, 0.0063, 0.1306, -0.0199, 0.0686, -0.1619, -0.0728, -0.1361, 0.1845, -0.0609, 0.0826, 0.0811, -0.0377, -0.2136, 0.0959, -0.0197, -0.0842, 0.1172, 0.09, -0.0601, 0.0546, 0.1051, 0.0147, -0.0316, 0.0336, 0.0307, -0.0185, 0.0436, -0.0498, 0.0, 0.1285, -0.0751, -0.0676, 0.0886, -0.2524, -0.2194, 0.0525, -0.2258, 0.0536, 0.0495, -0.0549, -0.131, 0.1235, 0.0052, -0.0627, -0.0184, 0.0112, 0.0199, 0.1696, 0.0641, 0.0145, 0.0344, 0.0872, 0.0322, -0.0893, -0.1592, -0.072, -0.1246, 0.1091, 0.1035, 0.0, -0.0268, -0.1875, -0.0082, -0.0001, -0.0684, -0.1668, 0.029, -0.1291, 0.1755, 0.0134, -0.0573, -0.07, -0.0221, -0.0012, 0.0937, 0.0263, -0.0426, 0.0216, 0.1419, 0.0081, -0.0624, -0.0817, -0.2448, -0.0748, 0.0581, 0.0554, 0.0196, 0.0298, 0.0275, 0.0955, 0.0833, 0.0, 0.0528, 0.0021, -0.0224, -0.0381, 0.0585, -0.0107, -0.0845, -0.1375, -0.0067, -0.045, -0.0339, -0.1751, -0.1792, 0.0223, -0.0307, 0.0692, 0.0295, -0.0948, -0.0528, 0.2141, -0.0708, -0.1652, -0.0884, -0.1298, 0.0763, 0.0304, -0.1137, -0.1175, 0.0322, 0.0419, 0.0491]}
        self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/HDGM2024_crust.cof",False,False,32),exp_dict)

        exp_dict ={'g': [0.0, -31543.0, -2298.0, -677.0, 2905.0, 924.0, 1022.0, -1469.0, 1256.0, 572.0, 876.0, 628.0, 660.0, -361.0, 134.0, -184.0, 328.0, 264.0, 5.0, -86.0, -16.0, 63.0, 61.0, -11.0, -217.0, -58.0, 59.0, -90.0, 70.0, -55.0, 0.0, 34.0, -41.0, -21.0, 18.0, 6.0, 11.0, 8.0, -4.0, -9.0, 1.0, 2.0, -9.0, 5.0, 8.0, 8.0, 10.0, 1.0, -11.0, 12.0, 1.0, -2.0, 2.0, -1.0, -1.0, -3.0, -4.0, 2.0, -5.0, -2.0, 6.0, 4.0, 0.0, 2.0, 2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'h': [0.0, 0.0, 5922.0, 0.0, -1061.0, 1121.0, 0.0, -330.0, 3.0, 523.0, 0.0, 195.0, -69.0, -210.0, -75.0, 0.0, -210.0, 53.0, -33.0, -124.0, 3.0, 0.0, -9.0, 83.0, 2.0, -35.0, 36.0, -69.0, 0.0, -45.0, -13.0, -10.0, -1.0, 28.0, -12.0, -22.0, 0.0, 8.0, -14.0, 7.0, -13.0, 5.0, 16.0, -5.0, -18.0, 0.0, -20.0, 14.0, 5.0, -3.0, -2.0, 8.0, 10.0, -2.0, 2.0, 0.0, 2.0, 1.0, 2.0, 6.0, -4.0, 0.0, -2.0, 4.0, 0.0, -6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'g_sv': [0.0, 15.8, 0.0, -10.2, 4.6, 23.4, 3.0, -5.0, -3.4, 12.6, 0.8, 3.0, -1.4, -3.8, 2.4, -1.6, 0.0, -1.0, -1.2, -1.4, -2.0, -0.2, -0.2, 0.0, -0.8, 0.2, -0.4, -0.4, 0.0, 0.2, 0.0, -0.2, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'h_sv': [0.0, 0.0, -2.6, 0.0, -5.0, -11.2, 0.0, -5.4, 6.2, -8.6, 0.0, 1.6, -1.6, 1.8, 2.0, 0.0, 3.4, 0.6, 0.2, -0.2, 1.6, 0.0, 0.4, 0.6, 0.4, 0.6, -0.8, 0.4, 0.0, -0.2, -0.2, -0.2, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, -0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],"epoch":1900}
 
        self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/HDGM2024_core.cof",False,True,None,1900),exp_dict)

        exp_dict = {'g': [0.0, -29775.0, -1848.0, -2131.0, 3059.0, 1686.0, 1314.0, -2239.0, 1248.0, 802.0, 939.0, 780.0, 325.0, -423.0, 141.0, -214.0, 353.0, 245.0, -109.0, -165.0, -36.0, 61.0, 65.0, 59.0, -178.0, 3.0, 18.0, -96.0, 77.0, -64.0, 2.0, 26.0, -1.0, 5.0, 9.0, 0.0, 23.0, 5.0, -1.0, -10.0, -12.0, 3.0, 4.0, 2.0, -6.0, 4.0, 9.0, 1.0, -12.0, 9.0, -4.0, -2.0, 7.0, 1.0, -6.0, -3.0, -4.0, 2.0, -5.0, -2.0, 4.0, 3.0, 1.0, 3.0, 3.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'h': [0.0, 0.0, 5406.0, 0.0, -2279.0, -373.0, 0.0, -284.0, 293.0, -352.0, 0.0, 247.0, -240.0, 84.0, -299.0, 0.0, 46.0, 154.0, -153.0, -69.0, 97.0, 0.0, -16.0, 82.0, 69.0, -52.0, 1.0, 24.0, 0.0, -80.0, -26.0, 0.0, 21.0, 17.0, -23.0, -4.0, 0.0, 10.0, -19.0, 6.0, -22.0, 12.0, 12.0, -16.0, -10.0, 0.0, -20.0, 15.0, 11.0, -7.0, -7.0, 9.0, 8.0, -7.0, 2.0, 0.0, 2.0, 1.0, 3.0, 6.0, -4.0, 0.0, -2.0, 3.0, -1.0, -6.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'g_sv': [0.0, 16.6, 12.8, -13.8, 2.2, -1.0, 4.2, -5.6, 0.2, -8.6, 0.2, 0.0, -7.0, 1.0, -3.8, 0.0, -0.2, -2.0, -1.8, -0.2, 3.8, 1.4, 0.4, 1.8, 1.6, -0.8, 0.2, 0.6, 0.0, -1.6, -0.2, 0.4, 1.2, -0.2, -0.2, -0.4, 0.4, 0.2, -1.0, 0.2, -0.4, 1.2, 0.4, -1.4, -0.2, 0.0, 0.0, 0.4, 0.4, -0.2, -0.8, 0.2, 0.6, -0.6, -0.4, 0.0, -0.4, 0.0, 0.2, 0.2, 0.0, -0.2, 0.2, 0.4, -0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0], 'h_sv': [0.0, 0.0, -20.0, 0.0, -17.4, -8.0, 0.0, 4.4, 1.8, -15.0, 0.0, 3.0, 0.8, 2.6, -1.4, 0.0, 0.0, 2.2, 2.0, 2.8, 2.0, 0.0, -0.2, -2.0, -0.4, -1.2, 0.0, 2.4, 0.0, 2.2, 0.2, 0.8, 0.6, 0.0, -0.2, -0.4, 0.0, 0.2, -0.4, 0.4, -0.2, 0.6, -0.2, 0.0, 1.2, 0.0, 0.0, 0.0, 0.2, 0.2, -0.2, -0.2, -0.6, -0.2, 0.2, 0.0, -0.2, -0.2, 0.2, -0.2, -0.2, -0.2, 0.0, -0.4, -0.2, -0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],"epoch":1990}

        self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/HDGM2024_core.cof",False,True,None,1990),exp_dict)

        exp_dict = {'g': [0.0, -29363.7462, -1421.0953, -2542.3755, 2956.36, 1657.789, 1363.4549, -2399.9783, 1243.2769, 468.978, 895.8471, 802.089, 61.5356, -286.669, 18.8091, -233.3177, 367.7999, 187.3489, -139.3211, -144.1348, 19.8663, 64.4544, 64.2214, 76.1193, -116.9897, -40.0744, 14.5841, -61.5994, 79.6185, -76.7823, -8.737, 58.7151, 15.9209, 3.1428, -10.2496, 13.4653, 23.0964, 10.6549, -17.4186, 1.4257, -21.6395, 16.6778, 14.8572, -16.7644, 0.6887, 4.7063, 8.0034, 2.9631, -0.4551, -2.267, -13.0939, 2.1766, 8.6799, -8.8233, -12.8057, -1.5615, -6.4003, 0.1577, 1.8741, -0.9724, -0.3343, -0.9489, 1.4987, 0.9433, -2.5958, -3.8198, 3.0004, -1.3762, -2.3991, 2.4103, -0.6787, 0.0318, -0.5515, -0.0595, 1.1657, -0.8191, -0.0404, 2.7236, -2.0753, -0.2055, 0.3345, 1.2168, -1.232, 0.5793, 0.4821, 0.4605, -0.0925, -0.4205, -0.1547, -1.2363, -0.7005, 0.1518, -0.9112, 0.6342, 0.7529, -0.2354, 0.5247, 0.1277, 0.6672, -0.0321, 0.3193, 0.1943, 0.4264, -0.5043, -0.3881, -0.2625, 0.2806, -0.2481, -0.0837, 0.0009, 0.1903, 0.0297, -0.1445, 0.2739, 0.0861, 0.6071, -0.2134, 0.4, 0.1851, -0.307, 0.1299, 0.5065, -0.007, 0.398, -0.2646, 0.12, -0.0642, -0.0482, 0.0738, -0.2399, -0.1629, 0.1738, -0.0121, -0.0123, 0.1314, -0.4356], 'h': [0.0, 0.0, 4566.0492, 0.0, -3107.1444, -803.8926, 0.0, -60.6233, 237.8227, -545.46, 0.0, 279.8592, -137.719, 210.5009, -371.3731, 0.0, 45.5689, 217.7825, -123.4251, 41.4376, 104.0289, 0.0, -18.4995, 18.4707, 49.2621, -60.601, 10.1671, 71.8346, 0.0, -49.3582, -14.8854, -0.1792, 23.4486, -6.4357, -25.6796, -2.0512, 0.0, 7.5696, -12.9927, 11.8757, -10.0745, 13.3252, 1.284, -5.4127, 3.861, 0.0, -24.3896, 12.0115, 8.6018, -3.622, -5.3303, 7.3631, -0.3649, 0.4076, 9.8267, 0.0, 3.3231, 0.0782, 2.6003, 5.332, -9.0143, 0.261, -4.2111, -3.7064, 0.6903, -9.0601, 0.0, 0.084, 2.7957, -0.5565, 0.0181, 0.5182, -0.3183, -1.284, -1.7396, -2.9367, -1.8773, -2.2861, 0.0, -1.1128, 0.6939, 1.1356, -1.4917, 0.0128, 0.6199, -0.1394, 0.758, 0.1374, -0.92, 0.0843, 0.2591, 0.0, -0.8709, 0.7651, 1.3078, -0.3322, -1.2521, -0.1288, 0.2226, -0.1807, 0.4664, 0.6025, -0.5727, -0.2741, -0.5015, 0.0, 0.4921, -0.5252, 0.3997, 0.6036, -0.1659, 0.2902, 0.4895, -0.0285, 0.4048, -0.0921, 0.2874, -0.1703, -0.2934, -0.1463, 0.0, 0.5525, 0.0243, 0.2415, 0.0794, 0.0344, 0.101, 0.2557, -0.1375, 0.1043, 0.025, -0.1733, -0.2165, -0.0098, -0.0884, 0.0996], 'g_sv': [0.0, 14.6405, 9.2329, -7.9432, -4.6332, -7.4227, -0.5314, -4.3862, 0.0704, -15.8186, -2.4846, -2.4187, -6.4086, 5.5368, -7.2193, 0.6763, 1.6861, 0.2211, 0.6609, 2.3798, 0.705, -0.0869, -0.3285, 0.9871, 1.1861, -0.8027, 0.427, 0.9406, -0.2892, -0.0043, -0.1882, 0.4278, -0.1381, -0.8592, -0.8011, 0.9158, -0.1325, 0.2186, 0.0425, 0.4936, -0.1768, 0.3058, 0.1781, -0.0648, 0.1847, -0.0948, -0.1294, 0.0767, 0.1964, -0.2589, 0.0556, 0.2836, -0.0412, 0.1665, -0.1843, 0.0747, -0.0663, 0.0891, -0.0046, -0.0144, -0.2531, -0.0082, -0.111, -0.0962, -0.0301, 0.032, -0.0048, -0.022, 0.0481, 0.0106, 0.0596, -0.0629, 0.0382, 0.0209, -0.078, -0.0487, -0.0524, -0.0956, -0.0286, -0.0401, -0.0416, -0.029, -0.0069, -0.0303, 0.0428, -0.0042, 0.0464, 0.0157, -0.077, -0.0185, -0.1077, 0.0059, 0.0029, 0.0206, 0.0051, 0.0184, -0.0653, 0.0354, -0.0323, 0.0046, -0.0187, 0.0183, -0.0143, -0.0008, 0.0098, 0.0101, -0.0042, -0.0023, 0.0053, 0.023, -0.0115, -0.0026, -0.0067, -0.0068, 0.0031, -0.0102, 0.0074, -0.0002, -0.0026, -0.016, -0.003, 0.0058, -0.0016, 0.0043, -0.001, -0.0004, 0.0035, 0.0059, -0.0026, 0.0039, 0.0016, -0.0029, 0.0016, 0.0024, 0.003, 0.0018], 'h_sv': [0.0, 0.0, -22.8191, 0.0, -29.0417, -12.6187, 0.0, 3.4327, -0.2959, -4.6385, 0.0, -1.142, 4.4308, 1.6132, -4.5647, 0.0, -0.8527, 1.9277, 0.3999, 1.7859, 1.842, 0.0, 0.5465, -1.3949, -0.3239, 0.9063, 0.6277, 0.9951, 0.0, 0.6333, 0.3881, -0.7166, 0.1123, -0.9565, 0.544, -0.2192, 0.0, -0.1947, 0.5071, -0.3096, 0.3421, -0.5148, -0.6592, 0.3107, 0.3213, 0.0, -0.1715, 0.2582, -0.2847, 0.3941, 0.239, -0.1573, -0.2201, 0.44, -0.0199, 0.0, -0.0381, 0.0852, -0.2167, 0.1302, -0.0843, 0.087, 0.0266, -0.0509, 0.1911, -0.0635, 0.0, 0.0395, 0.0691, -0.005, 0.0901, -0.0271, -0.0329, 0.0937, -0.0279, 0.0467, 0.0266, 0.0908, 0.0, 0.0184, 0.0565, -0.0451, 0.0808, -0.0107, -0.0275, 0.0067, 0.0494, -0.0014, 0.0073, 0.0386, -0.0768, 0.0, 0.0152, 0.0244, -0.0142, 0.0067, 0.0251, -0.0138, -0.0147, -0.0172, -0.0016, 0.0204, -0.0466, 0.0266, 0.0277, 0.0, 0.0203, 0.0168, 0.0133, 0.0078, -0.0079, -0.0106, 0.0158, -0.0048, -0.0011, -0.0125, 0.0046, -0.01, 0.0003, 0.0328, 0.0, -0.003, -0.0023, 0.0015, -0.0011, -0.0033, -0.0057, -0.004, 0.0025, -0.0036, -0.0007, -0.0006, 0.005, -0.0068, 0.0018, 0.0152], "epoch": 2024}

        self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/HDGM2024_core.cof",False,True,None,2024),exp_dict)

        exp_dict= {'g': [0,-36014.96, -2620.09, 888.44, 2168.45, -1761.04, 1517.59, -885.32, 897.32, -71.97, 411.49, 253.71, 309.28, 211.18, 218.44, 10.24, 115.43, 106.36, -207.31, 93.11, 314.34, 43.21, -14.21, 72.82, 16.4, -94.52, 73.7, 36.65, 16.85, -1.21, 8.55, 12.52, 0.12, -26.22, 26.96, -30.68, 1.26, 0.23, -1.59, 3.32, -0.24, -3.57, -8.81, 3.64, 0.44, 0.2, 0.05, -0.67, 0.5, 0.08, 0.47, -2.97, 1.25, -1.16, 2.79, 0.11, 0.14, -0.0, -0.23, 0.25, -0.29, 0.24, -0.36, 0.48, -0.61, -0.08, 0.05, 0.02, 0.08, -0.03, -0.01, -0.07, -0.08, 0.06, -0.04, 0.0, 0.07, -0.25, 0.02, 0.01, -0.0, 0.02, -0.02, -0.0, -0.04, -0.0, 0.0, -0.01, 0.0, -0.01, -0.03, -0.0, 0.01, -0.01, 0.0, 0.0, -0.0, 0.0, -0.01, 0.0, -0.01, -0.0, 0.0, -0.0, -0.0, -0.0, 0.0, 0.0, -0.0, 0.0, 0.0, 0.0, 0.0, -0.0, 0.0, -0.0, -0.0, -0.0, 0.0, -0.0, 14.0, -35999.94, -2639.44, 865.53, 2163.99, -1788.78, 1502.67, -879.45, 890.68, -75.24, 409.23, 257.16, 310.61, 212.15, 220.32, 11.82, 116.27, 106.7, -209.08, 92.42, 315.99, 43.19, -13.9, 72.75, 14.74, -94.05, 72.09, 37.34, 16.62, -1.06, 8.42, 12.45, -0.05], 'h': [0,0.0, 1225.51, 0.0, 2574.65, -3372.11, 0.0, -832.29, -287.17, -494.9, 0.0, -41.83, -707.57, -186.68, 126.51, 0.0, -5.81, -26.78, -58.64, -74.36, 167.9, 0.0, -13.6, 3.5, -10.06, -30.98, 1.87, 97.24, 0.0, -0.57, -6.65, -7.98, -10.68, 25.63, 2.58, -16.64, 0.0, 1.48, -2.53, -1.36, -5.28, 2.53, 8.28, -2.62, -21.16, 0.0, 0.47, -0.18, -0.73, -0.01, -2.01, 0.86, 1.97, -1.38, -2.56, 0.0, -0.02, 0.14, -0.24, 0.2, -0.24, -0.15, -0.14, 1.02, -0.94, 0.5, 0.0, 0.04, 0.04, 0.05, -0.03, 0.09, -0.08, 0.03, -0.0, 0.15, -0.21, 0.03, 0.0, 0.03, 0.0, 0.03, -0.01, -0.0, 0.0, 0.0, 0.03, -0.02, 0.03, 0.01, -0.01, 0.0, 0.01, 0.0, 0.0, 0.01, -0.0, 0.0, -0.0, 0.0, 0.01, -0.0, 0.02, 0.0, 0.01, 0.0, 0.0, 0.0, -0.0, 0.0, 0.0, -0.0, 0.0, -0.0, -0.0, -0.0, -0.0, 0.0, -0.0, 0.0, 14.0, 0.0, 1252.9, 0.0, 2559.65, -3356.48, 0.0, -831.87, -286.12, -490.7, 0.0, -39.07, -709.49, -186.29, 123.11, 0.0, -6.49, -26.06, -55.79, -73.11, 164.64, 0.0, -13.62, 3.54, -10.57, -29.46, 1.7, 96.59, 0.0, -0.56, -6.64, -8.04, -10.84], 'g_sv': [0,15.02, -19.35, -22.91, -4.46, -27.74, -14.93, 5.86, -6.63, -3.27, -2.25, 3.46, 1.34, 0.97, 1.88, 1.57, 0.84, 0.33, -1.77, -0.69, 1.65, -0.02, 0.32, -0.06, -1.65, 0.47, -1.61, 0.69, -0.23, 0.15, -0.14, -0.07, -0.17, 0.8, -0.58, 0.01, -0.04, 0.05, 0.02, 0.01, 0.01, 0.07, 0.16, -0.11, -0.17, -0.01, 0.0, 0.03, -0.0, -0.0, -0.01, 0.03, -0.0, -0.02, -0.04, -0.01, -0.0, 0.0, 0.01, -0.0, 0.0, -0.01, 0.01, -0.01, 0.0, 0.01, -0.0, -0.0, -0.0, 0.0, 0.0, 0.0, 0.0, -0.0, 0.0, -0.0, 0.0, 0.01, -0.0, -0.0, 0.0, -0.0, 0.0, 0.0, 0.0, 0.0, -0.0, 0.0, -0.0, 0.0, 0.0, 0.0, -0.0, -0.0, 0.0, 0.0, 0.0, -0.0, 0.0, -0.0, -0.0, 0.0, -0.0, -0.0, -0.0, 0.0, -0.0, -0.0, -0.0, 0.0, 0.0, 0.0, -0.0, 0.0, 0.0, 0.0, 0.0, 0.0, -0.0, 0.0, 0.0, 15.11, -20.39, -22.73, -4.45, -27.89, -14.85, 5.88, -6.67, -3.4, -2.23, 3.47, 1.33, 0.95, 1.82, 1.57, 0.85, 0.36, -1.78, -0.7, 1.64, -0.02, 0.31, -0.06, -1.65, 0.47, -1.61, 0.68, -0.23, 0.15, -0.14, -0.07, -0.17], 'h_sv': [0,0.0, 27.39, 0.0, -15.0, 15.63, 0.0, 0.41, 1.06, 4.2, 0.0, 2.76, -1.92, 0.38, -3.4, 0.0, -0.68, 0.72, 2.85, 1.25, -3.26, 0.0, -0.02, 0.04, -0.52, 1.53, -0.17, -0.65, 0.0, 0.02, 0.01, -0.06, -0.15, 0.22, -0.12, 0.74, 0.0, -0.05, 0.05, 0.01, 0.07, -0.03, -0.06, 0.09, 0.28, 0.0, 0.0, 0.01, 0.01, 0.01, 0.04, -0.01, -0.04, 0.02, 0.01, 0.0, 0.0, -0.0, 0.01, -0.0, 0.0, 0.0, 0.0, -0.01, 0.0, -0.01, 0.0, -0.0, -0.0, -0.0, 0.0, -0.0, -0.0, -0.0, 0.0, -0.0, 0.0, 0.0, 0.0, -0.0, -0.0, -0.0, 0.0, 0.0, -0.0, 0.0, -0.0, 0.0, -0.0, 0.0, 0.0, 0.0, -0.0, -0.0, 0.0, -0.0, 0.0, -0.0, 0.0, 0.0, 0.0, 0.0, -0.0, 0.0, -0.0, 0.0, -0.0, -0.0, -0.0, -0.0, -0.0, 0.0, -0.0, 0.0, -0.0, 0.0, -0.0, -0.0, -0.0, -0.0, 1591.0, 0.0, 25.16, 0.0, -15.87, 14.93, 0.0, 0.25, 0.78, 4.06, 0.0, 2.75, -2.0, 0.27, -3.45, 0.0, -0.67, 0.7, 2.82, 1.22, -3.25, 0.0, -0.02, 0.04, -0.53, 1.52, -0.17, -0.64, 0.0, 0.02, 0.01, -0.06, -0.15],"epoch": 1590}

        self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/IGRFPlus.COF",True,True, 16),exp_dict)

        exp_dict = {'g': [0,-29404.5, -1450.7, -2500.0, 2982.0, 1676.8], 'h': [0,0.0, 4652.9, 0.0, -2991.6, -734.8], 'g_sv': [0,6.7, 7.7, -11.5, -7.1, -2.2], 'h_sv': [0,0.0, -25.1, 0.0, -30.2, -23.9],"epoch":2020}

        #self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/WMM2020.COF",True,True, 2),exp_dict)
        

        #exp_dict = {'g': [0,-29404.5, -1450.7, -2500.0, 2982.0, 1676.8, 1363.9, -2381.0, 1236.2, 525.7, 903.1, 809.4, 86.2, -309.4, 47.9, -234.4, 363.1, 187.8, -140.7, -151.2, 13.7, 65.9, 65.6, 73.0, -121.5, -36.2, 13.5, -64.7, 80.6, -76.8, -8.3, 56.5, 15.8, 6.4, -7.2, 9.8], 'h': [0,0.0, 4652.9, 0.0, -2991.6, -734.8, 0.0, -82.2, 241.8, -542.9, 0.0, 282.0, -158.4, 199.8, -350.1, 0.0, 47.7, 208.4, -121.3, 32.2, 99.1, 0.0, -19.1, 25.0, 52.7, -64.4, 9.0, 68.1, 0.0, -51.4, -16.8, 2.3, 23.5, -2.2, -27.2, -1.9], 'g_sv': [0,6.7, 7.7, -11.5, -7.1, -2.2, 2.8, -6.2, 3.4, -12.2, -1.1, -1.6, -6.0, 5.4, -5.5, -0.3, 0.6, -0.7, 0.1, 1.2, 1.0, -0.6, -0.4, 0.5, 1.4, -1.4, -0.0, 0.8, -0.1, -0.3, -0.1, 0.7, 0.2, -0.5, -0.8, 1.0], 'h_sv': [0,0.0, -25.1, 0.0, -30.2, -23.9, 0.0, 5.7, -1.0, 1.1, 0.0, 0.2, 6.9, 3.7, -5.6, 0.0, 0.1, 2.5, -0.9, 3.0, 0.5, 0.0, 0.1, -1.8, -1.4, 0.9, 0.1, 1.0, 0.0, 0.5, 0.6, -0.7, -0.2, -1.2, 0.2, 0.3], "epoch":2020}
        #self._test_act_dict_to_exp_dict(sh_loader.load_coef("coefs/WMM2020.COF",True,True, 7),exp_dict)

    @unittest.skipIf(os.getenv("GITHUB_ACTIONS") == "true", "Skipping test in CI/CD since HDGM coefficient files are not allow to push in public")
    def test_timely_modify_magnetic_model(self):
        start_dict = sh_loader.load_coef("coefs/HDGM2024_core.cof",False,True,None, 2024)
        print(str(start_dict))
        act_dict = sh_loader.timely_modify_magnetic_model(start_dict, 2024)



        exp_dict = {"g":[0.000000,-29363.746200,-1421.095300,-2542.375500,2956.360000,1657.789000,1363.454900,-2399.978300,1243.276900,468.978000,895.847100,802.089000,61.535600,-286.669000,18.809100,-233.317700,367.799900,187.348900,-139.321100,-144.134800,19.866300,64.454400,64.221400,76.119300,-116.989700,-40.074400,14.584100,-61.599400,79.618500,-76.782300,-8.737000,58.715100,15.920900,3.142800,-10.249600,13.465300,23.096400,10.654900,-17.418600,1.425700,-21.639500,16.677800,14.857200,-16.764400,0.688700,4.706300,8.003400,2.963100,-0.455100,-2.267000,-13.093900,2.176600,8.679900,-8.823300,-12.805700,-1.561500,-6.400300,0.157700,1.874100,-0.972400,-0.334300,-0.948900,1.498700,0.943300,-2.595800,-3.819800,3.000400,-1.376200,-2.399100,2.410300,-0.678700,0.031800,-0.551500,-0.059500,1.165700,-0.819100,-0.040400,2.723600,-2.075300,-0.205500,0.334500,1.216800,-1.232000,0.579300,0.482100,0.460500,-0.092500,-0.420500,-0.154700,-1.236300,-0.700500,0.151800,-0.911200,0.634200,0.752900,-0.235400,0.524700,0.127700,0.667200,-0.032100,0.319300,0.194300,0.426400,-0.504300,-0.388100,-0.262500,0.280600,-0.248100,-0.083700,0.000900,0.190300,0.029700,-0.144500,0.273900,0.086100,0.607100,-0.213400,0.400000,0.185100,-0.307000,0.129900,0.506500,-0.007000,0.398000,-0.264600,0.120000,-0.064200,-0.048200,0.073800,-0.239900,-0.162900,0.173800,-0.012100,-0.012300,0.131400,-0.435600],"h":[0.000000,0.000000,4566.049200,0.000000,-3107.144400,-803.892600,0.000000,-60.623300,237.822700,-545.460000,0.000000,279.859200,-137.719000,210.500900,-371.373100,0.000000,45.568900,217.782500,-123.425100,41.437600,104.028900,0.000000,-18.499500,18.470700,49.262100,-60.601000,10.167100,71.834600,0.000000,-49.358200,-14.885400,-0.179200,23.448600,-6.435700,-25.679600,-2.051200,0.000000,7.569600,-12.992700,11.875700,-10.074500,13.325200,1.284000,-5.412700,3.861000,0.000000,-24.389600,12.011500,8.601800,-3.622000,-5.330300,7.363100,-0.364900,0.407600,9.826700,0.000000,3.323100,0.078200,2.600300,5.332000,-9.014300,0.261000,-4.211100,-3.706400,0.690300,-9.060100,0.000000,0.084000,2.795700,-0.556500,0.018100,0.518200,-0.318300,-1.284000,-1.739600,-2.936700,-1.877300,-2.286100,0.000000,-1.112800,0.693900,1.135600,-1.491700,0.012800,0.619900,-0.139400,0.758000,0.137400,-0.920000,0.084300,0.259100,0.000000,-0.870900,0.765100,1.307800,-0.332200,-1.252100,-0.128800,0.222600,-0.180700,0.466400,0.602500,-0.572700,-0.274100,-0.501500,0.000000,0.492100,-0.525200,0.399700,0.603600,-0.165900,0.290200,0.489500,-0.028500,0.404800,-0.092100,0.287400,-0.170300,-0.293400,-0.146300,0.000000,0.552500,0.024300,0.241500,0.079400,0.034400,0.101000,0.255700,-0.137500,0.104300,0.025000,-0.173300,-0.216500,-0.009800,-0.088400,0.099600],"g_sv":[0.000000,14.640500,9.232900,-7.943200,-4.633200,-7.422700,-0.531400,-4.386200,0.070400,-15.818600,-2.484600,-2.418700,-6.408600,5.536800,-7.219300,0.676300,1.686100,0.221100,0.660900,2.379800,0.705000,-0.086900,-0.328500,0.987100,1.186100,-0.802700,0.427000,0.940600,-0.289200,-0.004300,-0.188200,0.427800,-0.138100,-0.859200,-0.801100,0.915800,-0.132500,0.218600,0.042500,0.493600,-0.176800,0.305800,0.178100,-0.064800,0.184700,-0.094800,-0.129400,0.076700,0.196400,-0.258900,0.055600,0.283600,-0.041200,0.166500,-0.184300,0.074700,-0.066300,0.089100,-0.004600,-0.014400,-0.253100,-0.008200,-0.111000,-0.096200,-0.030100,0.032000,-0.004800,-0.022000,0.048100,0.010600,0.059600,-0.062900,0.038200,0.020900,-0.078000,-0.048700,-0.052400,-0.095600,-0.028600,-0.040100,-0.041600,-0.029000,-0.006900,-0.030300,0.042800,-0.004200,0.046400,0.015700,-0.077000,-0.018500,-0.107700,0.005900,0.002900,0.020600,0.005100,0.018400,-0.065300,0.035400,-0.032300,0.004600,-0.018700,0.018300,-0.014300,-0.000800,0.009800,0.010100,-0.004200,-0.002300,0.005300,0.023000,-0.011500,-0.002600,-0.006700,-0.006800,0.003100,-0.010200,0.007400,-0.000200,-0.002600,-0.016000,-0.003000,0.005800,-0.001600,0.004300,-0.001000,-0.000400,0.003500,0.005900,-0.002600,0.003900,0.001600,-0.002900,0.001600,0.002400,0.003000,0.001800],"h_sv":[0.000000,0.000000,-22.819100,0.000000,-29.041700,-12.618700,0.000000,3.432700,-0.295900,-4.638500,0.000000,-1.142000,4.430800,1.613200,-4.564700,0.000000,-0.852700,1.927700,0.399900,1.785900,1.842000,0.000000,0.546500,-1.394900,-0.323900,0.906300,0.627700,0.995100,0.000000,0.633300,0.388100,-0.716600,0.112300,-0.956500,0.544000,-0.219200,0.000000,-0.194700,0.507100,-0.309600,0.342100,-0.514800,-0.659200,0.310700,0.321300,0.000000,-0.171500,0.258200,-0.284700,0.394100,0.239000,-0.157300,-0.220100,0.440000,-0.019900,0.000000,-0.038100,0.085200,-0.216700,0.130200,-0.084300,0.087000,0.026600,-0.050900,0.191100,-0.063500,0.000000,0.039500,0.069100,-0.005000,0.090100,-0.027100,-0.032900,0.093700,-0.027900,0.046700,0.026600,0.090800,0.000000,0.018400,0.056500,-0.045100,0.080800,-0.010700,-0.027500,0.006700,0.049400,-0.001400,0.007300,0.038600,-0.076800,0.000000,0.015200,0.024400,-0.014200,0.006700,0.025100,-0.013800,-0.014700,-0.017200,-0.001600,0.020400,-0.046600,0.026600,0.027700,0.000000,0.020300,0.016800,0.013300,0.007800,-0.007900,-0.010600,0.015800,-0.004800,-0.001100,-0.012500,0.004600,-0.010000,0.000300,0.032800,0.000000,-0.003000,-0.002300,0.001500,-0.001100,-0.003300,-0.005700,-0.004000,0.002500,-0.003600,-0.000700,-0.000600,0.005000,-0.006800,0.001800,0.015200], "epoch": 2024}
        self._test_act_dict_to_exp_dict(act_dict,exp_dict)

        act_dict = sh_loader.timely_modify_magnetic_model(start_dict, 2024.304)

        exp_dict = {"g":[0.0000000000,-29359.2954880000,-1418.2884984000,-2544.7902328000,2954.9515072000,1655.5324992000,1363.2933544000,-2401.3117048000,1243.2983016000,464.1691456000,895.0917816000,801.3537152000,59.5873856000,-284.9858128000,16.6144328000,-233.1121048000,368.3124744000,187.4161144000,-139.1201864000,-143.4113408000,20.0806200000,64.4279824000,64.1215360000,76.4193784000,-116.6291256000,-40.3184208000,14.7139080000,-61.3134576000,79.5305832000,-76.7836072000,-8.7942128000,58.8451512000,15.8789176000,2.8816032000,-10.4931344000,13.7437032000,23.0561200000,10.7213544000,-17.4056800000,1.5757544000,-21.6932472000,16.7707632000,14.9113424000,-16.7840992000,0.7448488000,4.6774808000,7.9640624000,2.9864168000,-0.3953944000,-2.3457056000,-13.0769976000,2.2628144000,8.6673752000,-8.7726840000,-12.8617272000,-1.5387912000,-6.4204552000,0.1847864000,1.8727016000,-0.9767776000,-0.4112424000,-0.9513928000,1.4649560000,0.9140552000,-2.6049504000,-3.8100720000,2.9989408000,-1.3828880000,-2.3844776000,2.4135224000,-0.6605816000,0.0126784000,-0.5398872000,-0.0531464000,1.1419880000,-0.8339048000,-0.0563296000,2.6945376000,-2.0839944000,-0.2176904000,0.3218536000,1.2079840000,-1.2340976000,0.5700888000,0.4951112000,0.4592232000,-0.0783944000,-0.4157272000,-0.1781080000,-1.2419240000,-0.7332408000,0.1535936000,-0.9103184000,0.6404624000,0.7544504000,-0.2298064000,0.5048488000,0.1384616000,0.6573808000,-0.0307016000,0.3136152000,0.1998632000,0.4220528000,-0.5045432000,-0.3851208000,-0.2594296000,0.2793232000,-0.2487992000,-0.0820888000,0.0078920000,0.1868040000,0.0289096000,-0.1465368000,0.2718328000,0.0870424000,0.6039992000,-0.2111504000,0.3999392000,0.1843096000,-0.3118640000,0.1289880000,0.5082632000,-0.0074864000,0.3993072000,-0.2649040000,0.1198784000,-0.0631360000,-0.0464064000,0.0730096000,-0.2387144000,-0.1624136000,0.1729184000,-0.0116136000,-0.0115704000,0.1323120000,-0.4350528000],"h":[0.0000000000,0.0000000000,4559.1121936000,0.0000000000,-3115.9730768000,-807.7286848000,0.0000000000,-59.5797592000,237.7327464000,-546.8701040000,0.0000000000,279.5120320000,-136.3720368000,210.9913128000,-372.7607688000,0.0000000000,45.3096792000,218.3685208000,-123.3035304000,41.9805136000,104.5888680000,0.0000000000,-18.3333640000,18.0466504000,49.1636344000,-60.3254848000,10.3579208000,72.1371104000,0.0000000000,-49.1656768000,-14.7674176000,-0.3970464000,23.4827392000,-6.7264760000,-25.5142240000,-2.1178368000,0.0000000000,7.5104112000,-12.8385416000,11.7815816000,-9.9705016000,13.1687008000,1.0836032000,-5.3182472000,3.9586752000,0.0000000000,-24.4417360000,12.0899928000,8.5152512000,-3.5021936000,-5.2576440000,7.3152808000,-0.4318104000,0.5413600000,9.8206504000,0.0000000000,3.3115176000,0.1041008000,2.5344232000,5.3715808000,-9.0399272000,0.2874480000,-4.2030136000,-3.7218736000,0.7483944000,-9.0794040000,0.0000000000,0.0960080000,2.8167064000,-0.5580200000,0.0454904000,0.5099616000,-0.3283016000,-1.2555152000,-1.7480816000,-2.9225032000,-1.8692136000,-2.2584968000,0.0000000000,-1.1072064000,0.7110760000,1.1218896000,-1.4671368000,0.0095472000,0.6115400000,-0.1373632000,0.7730176000,0.1369744000,-0.9177808000,0.0960344000,0.2357528000,0.0000000000,-0.8662792000,0.7725176000,1.3034832000,-0.3301632000,-1.2444696000,-0.1329952000,0.2181312000,-0.1859288000,0.4659136000,0.6087016000,-0.5868664000,-0.2660136000,-0.4930792000,0.0000000000,0.4982712000,-0.5200928000,0.4037432000,0.6059712000,-0.1683016000,0.2869776000,0.4943032000,-0.0299592000,0.4044656000,-0.0959000000,0.2887984000,-0.1733400000,-0.2933088000,-0.1363288000,0.0000000000,0.5515880000,0.0236008000,0.2419560000,0.0790656000,0.0333968000,0.0992672000,0.2544840000,-0.1367400000,0.1032056000,0.0247872000,-0.1734824000,-0.2149800000,-0.0118672000,-0.0878528000,0.1042208000],"g_sv":[0.0000000000,14.6405000000,9.2329000000,-7.9432000000,-4.6332000000,-7.4227000000,-0.5314000000,-4.3862000000,0.0704000000,-15.8186000000,-2.4846000000,-2.4187000000,-6.4086000000,5.5368000000,-7.2193000000,0.6763000000,1.6861000000,0.2211000000,0.6609000000,2.3798000000,0.7050000000,-0.0869000000,-0.3285000000,0.9871000000,1.1861000000,-0.8027000000,0.4270000000,0.9406000000,-0.2892000000,-0.0043000000,-0.1882000000,0.4278000000,-0.1381000000,-0.8592000000,-0.8011000000,0.9158000000,-0.1325000000,0.2186000000,0.0425000000,0.4936000000,-0.1768000000,0.3058000000,0.1781000000,-0.0648000000,0.1847000000,-0.0948000000,-0.1294000000,0.0767000000,0.1964000000,-0.2589000000,0.0556000000,0.2836000000,-0.0412000000,0.1665000000,-0.1843000000,0.0747000000,-0.0663000000,0.0891000000,-0.0046000000,-0.0144000000,-0.2531000000,-0.0082000000,-0.1110000000,-0.0962000000,-0.0301000000,0.0320000000,-0.0048000000,-0.0220000000,0.0481000000,0.0106000000,0.0596000000,-0.0629000000,0.0382000000,0.0209000000,-0.0780000000,-0.0487000000,-0.0524000000,-0.0956000000,-0.0286000000,-0.0401000000,-0.0416000000,-0.0290000000,-0.0069000000,-0.0303000000,0.0428000000,-0.0042000000,0.0464000000,0.0157000000,-0.0770000000,-0.0185000000,-0.1077000000,0.0059000000,0.0029000000,0.0206000000,0.0051000000,0.0184000000,-0.0653000000,0.0354000000,-0.0323000000,0.0046000000,-0.0187000000,0.0183000000,-0.0143000000,-0.0008000000,0.0098000000,0.0101000000,-0.0042000000,-0.0023000000,0.0053000000,0.0230000000,-0.0115000000,-0.0026000000,-0.0067000000,-0.0068000000,0.0031000000,-0.0102000000,0.0074000000,-0.0002000000,-0.0026000000,-0.0160000000,-0.0030000000,0.0058000000,-0.0016000000,0.0043000000,-0.0010000000,-0.0004000000,0.0035000000,0.0059000000,-0.0026000000,0.0039000000,0.0016000000,-0.0029000000,0.0016000000,0.0024000000,0.0030000000,0.0018000000],"h_sv":[0.0000000000,0.0000000000,-22.8191000000,0.0000000000,-29.0417000000,-12.6187000000,0.0000000000,3.4327000000,-0.2959000000,-4.6385000000,0.0000000000,-1.1420000000,4.4308000000,1.6132000000,-4.5647000000,0.0000000000,-0.8527000000,1.9277000000,0.3999000000,1.7859000000,1.8420000000,0.0000000000,0.5465000000,-1.3949000000,-0.3239000000,0.9063000000,0.6277000000,0.9951000000,0.0000000000,0.6333000000,0.3881000000,-0.7166000000,0.1123000000,-0.9565000000,0.5440000000,-0.2192000000,0.0000000000,-0.1947000000,0.5071000000,-0.3096000000,0.3421000000,-0.5148000000,-0.6592000000,0.3107000000,0.3213000000,0.0000000000,-0.1715000000,0.2582000000,-0.2847000000,0.3941000000,0.2390000000,-0.1573000000,-0.2201000000,0.4400000000,-0.0199000000,0.0000000000,-0.0381000000,0.0852000000,-0.2167000000,0.1302000000,-0.0843000000,0.0870000000,0.0266000000,-0.0509000000,0.1911000000,-0.0635000000,0.0000000000,0.0395000000,0.0691000000,-0.0050000000,0.0901000000,-0.0271000000,-0.0329000000,0.0937000000,-0.0279000000,0.0467000000,0.0266000000,0.0908000000,0.0000000000,0.0184000000,0.0565000000,-0.0451000000,0.0808000000,-0.0107000000,-0.0275000000,0.0067000000,0.0494000000,-0.0014000000,0.0073000000,0.0386000000,-0.0768000000,0.0000000000,0.0152000000,0.0244000000,-0.0142000000,0.0067000000,0.0251000000,-0.0138000000,-0.0147000000,-0.0172000000,-0.0016000000,0.0204000000,-0.0466000000,0.0266000000,0.0277000000,0.0000000000,0.0203000000,0.0168000000,0.0133000000,0.0078000000,-0.0079000000,-0.0106000000,0.0158000000,-0.0048000000,-0.0011000000,-0.0125000000,0.0046000000,-0.0100000000,0.0003000000,0.0328000000,0.0000000000,-0.0030000000,-0.0023000000,0.0015000000,-0.0011000000,-0.0033000000,-0.0057000000,-0.0040000000,0.0025000000,-0.0036000000,-0.0007000000,-0.0006000000,0.0050000000,-0.0068000000,0.0018000000,0.0152000000],"epoch":2024}

        self._test_act_dict_to_exp_dict(act_dict,exp_dict)

        act_dict = sh_loader.timely_modify_magnetic_model(start_dict, 2024.754)
        exp_dict = {"g":[0.0000000000,-29352.7072630000,-1414.1336934000,-2548.3646728000,2952.8665672000,1652.1922842000,1363.0542244000,-2403.2854948000,1243.3299816000,457.0507756000,893.9737116000,800.2653002000,56.7035156000,-282.4942528000,13.3657478000,-232.8077698000,369.0712194000,187.5156094000,-138.8227814000,-142.3404308000,20.3978700000,64.3888774000,63.9737110000,76.8635734000,-116.0953806000,-40.6796358000,14.9060580000,-60.8901876000,79.4004432000,-76.7855422000,-8.8789028000,59.0376612000,15.8167726000,2.4949632000,-10.8536294000,14.1558132000,22.9964950000,10.8197244000,-17.3865550000,1.7978744000,-21.7728072000,16.9083732000,14.9914874000,-16.8132592000,0.8279638000,4.6348208000,7.9058324000,3.0209318000,-0.3070144000,-2.4622106000,-13.0519776000,2.3904344000,8.6488352000,-8.6977590000,-12.9446622000,-1.5051762000,-6.4502902000,0.2248814000,1.8706316000,-0.9832576000,-0.5251374000,-0.9550828000,1.4150060000,0.8707652000,-2.6184954000,-3.7956720000,2.9967808000,-1.3927880000,-2.3628326000,2.4182924000,-0.6337616000,-0.0156266000,-0.5226972000,-0.0437414000,1.1068880000,-0.8558198000,-0.0799096000,2.6515176000,-2.0968644000,-0.2357354000,0.3031336000,1.1949340000,-1.2372026000,0.5564538000,0.5143712000,0.4573332000,-0.0575144000,-0.4086622000,-0.2127580000,-1.2502490000,-0.7817058000,0.1562486000,-0.9090134000,0.6497324000,0.7567454000,-0.2215264000,0.4754638000,0.1543916000,0.6428458000,-0.0286316000,0.3052002000,0.2080982000,0.4156178000,-0.5049032000,-0.3807108000,-0.2548846000,0.2774332000,-0.2498342000,-0.0797038000,0.0182420000,0.1816290000,0.0277396000,-0.1495518000,0.2687728000,0.0884374000,0.5994092000,-0.2078204000,0.3998492000,0.1831396000,-0.3190640000,0.1276380000,0.5108732000,-0.0082064000,0.4012422000,-0.2653540000,0.1196984000,-0.0615610000,-0.0437514000,0.0718396000,-0.2369594000,-0.1616936000,0.1716134000,-0.0108936000,-0.0104904000,0.1336620000,-0.4342428000],"h":[0.0000000000,0.0000000000,4548.8435986000,0.0000000000,-3129.0418418000,-813.4070998000,0.0000000000,-58.0350442000,237.5995914000,-548.9574290000,0.0000000000,278.9981320000,-134.3781768000,211.7172528000,-374.8148838000,0.0000000000,44.9259642000,219.2359858000,-123.1235754000,42.7841686000,105.4177680000,0.0000000000,-18.0874390000,17.4189454000,49.0178794000,-59.9176498000,10.6403858000,72.5849054000,0.0000000000,-48.8806918000,-14.5927726000,-0.7195164000,23.5332742000,-7.1569010000,-25.2694240000,-2.2164768000,0.0000000000,7.4227962000,-12.6103466000,11.6422616000,-9.8165566000,12.9370408000,0.7869632000,-5.1784322000,4.1032602000,0.0000000000,-24.5189110000,12.2061828000,8.3871362000,-3.3248486000,-5.1500940000,7.2444958000,-0.5308554000,0.7393600000,9.8116954000,0.0000000000,3.2943726000,0.1424408000,2.4369082000,5.4301708000,-9.0778622000,0.3265980000,-4.1910436000,-3.7447786000,0.8343894000,-9.1079790000,0.0000000000,0.1137830000,2.8478014000,-0.5602700000,0.0860354000,0.4977666000,-0.3431066000,-1.2133502000,-1.7606366000,-2.9014882000,-1.8572436000,-2.2176368000,0.0000000000,-1.0989264000,0.7365010000,1.1015946000,-1.4307768000,0.0047322000,0.5991650000,-0.1343482000,0.7952476000,0.1363444000,-0.9144958000,0.1134044000,0.2011928000,0.0000000000,-0.8594392000,0.7834976000,1.2970932000,-0.3271482000,-1.2331746000,-0.1392052000,0.2115162000,-0.1936688000,0.4651936000,0.6178816000,-0.6078364000,-0.2540436000,-0.4806142000,0.0000000000,0.5074062000,-0.5125328000,0.4097282000,0.6094812000,-0.1718566000,0.2822076000,0.5014132000,-0.0321192000,0.4039706000,-0.1015250000,0.2908684000,-0.1778400000,-0.2931738000,-0.1215688000,0.0000000000,0.5502380000,0.0225658000,0.2426310000,0.0785706000,0.0319118000,0.0967022000,0.2526840000,-0.1356150000,0.1015856000,0.0244722000,-0.1737524000,-0.2127300000,-0.0149272000,-0.0870428000,0.1110608000],"g_sv":[0.0000000000,14.6405000000,9.2329000000,-7.9432000000,-4.6332000000,-7.4227000000,-0.5314000000,-4.3862000000,0.0704000000,-15.8186000000,-2.4846000000,-2.4187000000,-6.4086000000,5.5368000000,-7.2193000000,0.6763000000,1.6861000000,0.2211000000,0.6609000000,2.3798000000,0.7050000000,-0.0869000000,-0.3285000000,0.9871000000,1.1861000000,-0.8027000000,0.4270000000,0.9406000000,-0.2892000000,-0.0043000000,-0.1882000000,0.4278000000,-0.1381000000,-0.8592000000,-0.8011000000,0.9158000000,-0.1325000000,0.2186000000,0.0425000000,0.4936000000,-0.1768000000,0.3058000000,0.1781000000,-0.0648000000,0.1847000000,-0.0948000000,-0.1294000000,0.0767000000,0.1964000000,-0.2589000000,0.0556000000,0.2836000000,-0.0412000000,0.1665000000,-0.1843000000,0.0747000000,-0.0663000000,0.0891000000,-0.0046000000,-0.0144000000,-0.2531000000,-0.0082000000,-0.1110000000,-0.0962000000,-0.0301000000,0.0320000000,-0.0048000000,-0.0220000000,0.0481000000,0.0106000000,0.0596000000,-0.0629000000,0.0382000000,0.0209000000,-0.0780000000,-0.0487000000,-0.0524000000,-0.0956000000,-0.0286000000,-0.0401000000,-0.0416000000,-0.0290000000,-0.0069000000,-0.0303000000,0.0428000000,-0.0042000000,0.0464000000,0.0157000000,-0.0770000000,-0.0185000000,-0.1077000000,0.0059000000,0.0029000000,0.0206000000,0.0051000000,0.0184000000,-0.0653000000,0.0354000000,-0.0323000000,0.0046000000,-0.0187000000,0.0183000000,-0.0143000000,-0.0008000000,0.0098000000,0.0101000000,-0.0042000000,-0.0023000000,0.0053000000,0.0230000000,-0.0115000000,-0.0026000000,-0.0067000000,-0.0068000000,0.0031000000,-0.0102000000,0.0074000000,-0.0002000000,-0.0026000000,-0.0160000000,-0.0030000000,0.0058000000,-0.0016000000,0.0043000000,-0.0010000000,-0.0004000000,0.0035000000,0.0059000000,-0.0026000000,0.0039000000,0.0016000000,-0.0029000000,0.0016000000,0.0024000000,0.0030000000,0.0018000000],"h_sv":[0.0000000000,0.0000000000,-22.8191000000,0.0000000000,-29.0417000000,-12.6187000000,0.0000000000,3.4327000000,-0.2959000000,-4.6385000000,0.0000000000,-1.1420000000,4.4308000000,1.6132000000,-4.5647000000,0.0000000000,-0.8527000000,1.9277000000,0.3999000000,1.7859000000,1.8420000000,0.0000000000,0.5465000000,-1.3949000000,-0.3239000000,0.9063000000,0.6277000000,0.9951000000,0.0000000000,0.6333000000,0.3881000000,-0.7166000000,0.1123000000,-0.9565000000,0.5440000000,-0.2192000000,0.0000000000,-0.1947000000,0.5071000000,-0.3096000000,0.3421000000,-0.5148000000,-0.6592000000,0.3107000000,0.3213000000,0.0000000000,-0.1715000000,0.2582000000,-0.2847000000,0.3941000000,0.2390000000,-0.1573000000,-0.2201000000,0.4400000000,-0.0199000000,0.0000000000,-0.0381000000,0.0852000000,-0.2167000000,0.1302000000,-0.0843000000,0.0870000000,0.0266000000,-0.0509000000,0.1911000000,-0.0635000000,0.0000000000,0.0395000000,0.0691000000,-0.0050000000,0.0901000000,-0.0271000000,-0.0329000000,0.0937000000,-0.0279000000,0.0467000000,0.0266000000,0.0908000000,0.0000000000,0.0184000000,0.0565000000,-0.0451000000,0.0808000000,-0.0107000000,-0.0275000000,0.0067000000,0.0494000000,-0.0014000000,0.0073000000,0.0386000000,-0.0768000000,0.0000000000,0.0152000000,0.0244000000,-0.0142000000,0.0067000000,0.0251000000,-0.0138000000,-0.0147000000,-0.0172000000,-0.0016000000,0.0204000000,-0.0466000000,0.0266000000,0.0277000000,0.0000000000,0.0203000000,0.0168000000,0.0133000000,0.0078000000,-0.0079000000,-0.0106000000,0.0158000000,-0.0048000000,-0.0011000000,-0.0125000000,0.0046000000,-0.0100000000,0.0003000000,0.0328000000,0.0000000000,-0.0030000000,-0.0023000000,0.0015000000,-0.0011000000,-0.0033000000,-0.0057000000,-0.0040000000,0.0025000000,-0.0036000000,-0.0007000000,-0.0006000000,0.0050000000,-0.0068000000,0.0018000000,0.0152000000], "epoch": 2024}

        

        self._test_act_dict_to_exp_dict(act_dict,exp_dict)

        act_dict = sh_loader.timely_modify_magnetic_model(start_dict, 2025.7)
        exp_dict = {"g":[0.0000000000,-29338.8573500000,-1405.3993700000,-2555.8789400000,2948.4835600000,1645.1704100000,1362.5515200000,-2407.4348400000,1243.3965800000,442.0863800000,891.6232800000,797.9772100000,50.6409800000,-277.2564400000,6.5362900000,-232.1679900000,370.6662700000,187.7247700000,-138.1975700000,-140.0891400000,21.0648000000,64.3066700000,63.6629500000,77.7973700000,-114.9733300000,-41.4389900000,15.3100000000,-60.0003800000,79.1268600000,-76.7896100000,-9.0569400000,59.4423600000,15.6861300000,1.6821600000,-11.6114700000,15.0221600000,22.8711500000,11.0265200000,-17.3463500000,2.2648200000,-21.9400600000,17.1976600000,15.1599700000,-16.8745600000,1.0026900000,4.5451400000,7.7834200000,3.0934900000,-0.1212200000,-2.7071300000,-12.9993800000,2.6587200000,8.6098600000,-8.5402500000,-13.1190100000,-1.4345100000,-6.5130100000,0.3091700000,1.8662800000,-0.9968800000,-0.7645700000,-0.9628400000,1.3100000000,0.7797600000,-2.6469700000,-3.7654000000,2.9922400000,-1.4136000000,-2.3173300000,2.4283200000,-0.5773800000,-0.0751300000,-0.4865600000,-0.0239700000,1.0331000000,-0.9018900000,-0.1294800000,2.5610800000,-2.1239200000,-0.2736700000,0.2637800000,1.1675000000,-1.2437300000,0.5277900000,0.5548600000,0.4533600000,-0.0136200000,-0.3938100000,-0.2856000000,-1.2677500000,-0.8835900000,0.1618300000,-0.9062700000,0.6692200000,0.7615700000,-0.2041200000,0.4136900000,0.1878800000,0.6122900000,-0.0242800000,0.2875100000,0.2254100000,0.4020900000,-0.5056600000,-0.3714400000,-0.2453300000,0.2734600000,-0.2520100000,-0.0746900000,0.0400000000,0.1707500000,0.0252800000,-0.1558900000,0.2623400000,0.0913700000,0.5897600000,-0.2008200000,0.3996600000,0.1806800000,-0.3342000000,0.1248000000,0.5163600000,-0.0097200000,0.4053100000,-0.2663000000,0.1193200000,-0.0582500000,-0.0381700000,0.0693800000,-0.2332700000,-0.1601800000,0.1688700000,-0.0093800000,-0.0082200000,0.1365000000,-0.4325400000],"h":[0.0000000000,0.0000000000,4527.2567300000,0.0000000000,-3156.5152900000,-825.3443900000,0.0000000000,-54.7877100000,237.3196700000,-553.3454500000,0.0000000000,277.9178000000,-130.1866400000,213.2433400000,-379.1330900000,0.0000000000,44.1193100000,221.0595900000,-122.7452700000,44.4736300000,107.1603000000,0.0000000000,-17.5704500000,16.0993700000,48.7114700000,-59.0602900000,11.2341900000,73.5262700000,0.0000000000,-48.2815900000,-14.2256300000,-1.3974200000,23.6395100000,-8.0617500000,-24.7548000000,-2.4238400000,0.0000000000,7.2386100000,-12.1306300000,11.3493800000,-9.4929300000,12.4500400000,0.1633600000,-4.8845100000,4.4072100000,0.0000000000,-24.6811500000,12.4504400000,8.1178100000,-2.9520300000,-4.9240000000,7.0956900000,-0.7390700000,1.1556000000,9.7928700000,0.0000000000,3.2583300000,0.2230400000,2.2319100000,5.5533400000,-9.1576100000,0.4089000000,-4.1658800000,-3.7929300000,1.0151700000,-9.1680500000,0.0000000000,0.1511500000,2.9131700000,-0.5650000000,0.1712700000,0.4721300000,-0.3742300000,-1.1247100000,-1.7870300000,-2.8573100000,-1.8320800000,-2.1317400000,0.0000000000,-1.0815200000,0.7899500000,1.0589300000,-1.3543400000,-0.0053900000,0.5731500000,-0.1280100000,0.8419800000,0.1350200000,-0.9075900000,0.1499200000,0.1285400000,0.0000000000,-0.8450600000,0.8065800000,1.2836600000,-0.3208100000,-1.2094300000,-0.1522600000,0.1976100000,-0.2099400000,0.4636800000,0.6371800000,-0.6519200000,-0.2288800000,-0.4544100000,0.0000000000,0.5266100000,-0.4966400000,0.4223100000,0.6168600000,-0.1793300000,0.2721800000,0.5163600000,-0.0366600000,0.4029300000,-0.1133500000,0.2952200000,-0.1873000000,-0.2928900000,-0.0905400000,0.0000000000,0.5474000000,0.0203900000,0.2440500000,0.0775300000,0.0287900000,0.0913100000,0.2489000000,-0.1332500000,0.0981800000,0.0238100000,-0.1743200000,-0.2080000000,-0.0213600000,-0.0853400000,0.1254400000],"g_sv":[0.0000000000,14.6405000000,9.2329000000,-7.9432000000,-4.6332000000,-7.4227000000,-0.5314000000,-4.3862000000,0.0704000000,-15.8186000000,-2.4846000000,-2.4187000000,-6.4086000000,5.5368000000,-7.2193000000,0.6763000000,1.6861000000,0.2211000000,0.6609000000,2.3798000000,0.7050000000,-0.0869000000,-0.3285000000,0.9871000000,1.1861000000,-0.8027000000,0.4270000000,0.9406000000,-0.2892000000,-0.0043000000,-0.1882000000,0.4278000000,-0.1381000000,-0.8592000000,-0.8011000000,0.9158000000,-0.1325000000,0.2186000000,0.0425000000,0.4936000000,-0.1768000000,0.3058000000,0.1781000000,-0.0648000000,0.1847000000,-0.0948000000,-0.1294000000,0.0767000000,0.1964000000,-0.2589000000,0.0556000000,0.2836000000,-0.0412000000,0.1665000000,-0.1843000000,0.0747000000,-0.0663000000,0.0891000000,-0.0046000000,-0.0144000000,-0.2531000000,-0.0082000000,-0.1110000000,-0.0962000000,-0.0301000000,0.0320000000,-0.0048000000,-0.0220000000,0.0481000000,0.0106000000,0.0596000000,-0.0629000000,0.0382000000,0.0209000000,-0.0780000000,-0.0487000000,-0.0524000000,-0.0956000000,-0.0286000000,-0.0401000000,-0.0416000000,-0.0290000000,-0.0069000000,-0.0303000000,0.0428000000,-0.0042000000,0.0464000000,0.0157000000,-0.0770000000,-0.0185000000,-0.1077000000,0.0059000000,0.0029000000,0.0206000000,0.0051000000,0.0184000000,-0.0653000000,0.0354000000,-0.0323000000,0.0046000000,-0.0187000000,0.0183000000,-0.0143000000,-0.0008000000,0.0098000000,0.0101000000,-0.0042000000,-0.0023000000,0.0053000000,0.0230000000,-0.0115000000,-0.0026000000,-0.0067000000,-0.0068000000,0.0031000000,-0.0102000000,0.0074000000,-0.0002000000,-0.0026000000,-0.0160000000,-0.0030000000,0.0058000000,-0.0016000000,0.0043000000,-0.0010000000,-0.0004000000,0.0035000000,0.0059000000,-0.0026000000,0.0039000000,0.0016000000,-0.0029000000,0.0016000000,0.0024000000,0.0030000000,0.0018000000],"h_sv":[0.0000000000,0.0000000000,-22.8191000000,0.0000000000,-29.0417000000,-12.6187000000,0.0000000000,3.4327000000,-0.2959000000,-4.6385000000,0.0000000000,-1.1420000000,4.4308000000,1.6132000000,-4.5647000000,0.0000000000,-0.8527000000,1.9277000000,0.3999000000,1.7859000000,1.8420000000,0.0000000000,0.5465000000,-1.3949000000,-0.3239000000,0.9063000000,0.6277000000,0.9951000000,0.0000000000,0.6333000000,0.3881000000,-0.7166000000,0.1123000000,-0.9565000000,0.5440000000,-0.2192000000,0.0000000000,-0.1947000000,0.5071000000,-0.3096000000,0.3421000000,-0.5148000000,-0.6592000000,0.3107000000,0.3213000000,0.0000000000,-0.1715000000,0.2582000000,-0.2847000000,0.3941000000,0.2390000000,-0.1573000000,-0.2201000000,0.4400000000,-0.0199000000,0.0000000000,-0.0381000000,0.0852000000,-0.2167000000,0.1302000000,-0.0843000000,0.0870000000,0.0266000000,-0.0509000000,0.1911000000,-0.0635000000,0.0000000000,0.0395000000,0.0691000000,-0.0050000000,0.0901000000,-0.0271000000,-0.0329000000,0.0937000000,-0.0279000000,0.0467000000,0.0266000000,0.0908000000,0.0000000000,0.0184000000,0.0565000000,-0.0451000000,0.0808000000,-0.0107000000,-0.0275000000,0.0067000000,0.0494000000,-0.0014000000,0.0073000000,0.0386000000,-0.0768000000,0.0000000000,0.0152000000,0.0244000000,-0.0142000000,0.0067000000,0.0251000000,-0.0138000000,-0.0147000000,-0.0172000000,-0.0016000000,0.0204000000,-0.0466000000,0.0266000000,0.0277000000,0.0000000000,0.0203000000,0.0168000000,0.0133000000,0.0078000000,-0.0079000000,-0.0106000000,0.0158000000,-0.0048000000,-0.0011000000,-0.0125000000,0.0046000000,-0.0100000000,0.0003000000,0.0328000000,0.0000000000,-0.0030000000,-0.0023000000,0.0015000000,-0.0011000000,-0.0033000000,-0.0057000000,-0.0040000000,0.0025000000,-0.0036000000,-0.0007000000,-0.0006000000,0.0050000000,-0.0068000000,0.0018000000,0.0152000000], "epoch": 2024 }


        self._test_act_dict_to_exp_dict(act_dict,exp_dict)

   




if __name__ == '__main__':
    unittest.main()

